@@ -20,6 +20,7 @@
 
 #include "value.h"
 #include "utils.h"
+#include "callbacks.h"
 
 namespace dht {
 
@@ -44,7 +45,14 @@ struct Listener {
 struct LocalListener {
     Sp<Query> query;
     Value::Filter filter;
-    GetCallback get_cb;
+    ValueCallback get_cb;
+};
+
+
+struct SearchListener {
+    Sp<Query> query;
+    Value::Filter filter;
+    ValueCallback get_cb;
 };
 
 }
